@@ -1,16 +1,20 @@
 package JE::LValue;
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 use strict;
 use warnings;
 
+use Scalar::Util 'blessed';
+
 # ~~~ I need to figure out how to delegate overloaded ops to the object
 #     contained (referenced) by the JE::LValue, such that $lvalue->{0}
 #     will produce the same as $lvalue->get->{0}. I may need to make
 #     JE::LValue objects into references to array refs \[ ... ]. Right
 #     now they are just array refs [ ... ].
 
+# ~~~ We probably need a C<can> method.
+
 #use overload nomethod => sub {
 #	
 #};
@@ -23,15 +27,30 @@ sub new {
 
 sub get {
 	my $self = shift;
-	$self->[0]->prop($self->[1]);
+	my $val = $self->[0]->prop($self->[1]);
+	defined $val ? $val : $self->[0]->global->undefined;
+		# If we have a Perl undef, then the property does not
+		# not exist, and we have to return a JS undefined val.
 }
 
 sub set {
 	my $obj = (my $self = shift)->[0];
-	$obj->id eq 'null' and $obj = $$obj;
+	$obj->id eq 'null' and $obj = $$self[0] = $$obj;
 	$obj->prop($self->[1], shift);
+	$self;
 }
 
+sub call {
+	my $base_obj = (my $self = shift)->[0];
+	my $prop = $self->get;
+	blessed $prop and can $prop 'apply' or die; # ~~~ TypeError, I fink
+	$prop->apply($base_obj, @_);
+}
+
+sub base { shift->[0] }
+
+sub property { shift->[1] }
+
 our $AUTOLOAD;
 
 sub AUTOLOAD {
@@ -86,7 +105,20 @@ Gets the value of the property.
 
 =item $lv->set($value)
 
-Sets the property to $value.
+Sets the property to $value and returns $lv.
+
+=item $lv->call(@args)
+
+If the property is a function, this calls the function with the
+base object as the 'this' value.
+
+=item $lv->base
+
+Returns the base object.
+
+=item $lv->property
+
+Returns the property name.
 
 =cut
 
