@@ -1,14 +1,15 @@
 package JE::Object::Function;
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 
 use strict;
 use warnings;
 
+use Scalar::Util 'blessed';
+
 our @ISA = 'JE::Object';
 
-require JE::Object::Function::Call;
 require JE::Object;
 require JE::Scope;
 require JE::Code;
@@ -189,7 +190,8 @@ discarded, and the (possibly modified) object will be returned.
 
 =item constructor_args
 
-Like C<function_args>, but the C<'this'> string does not apply. If it is
+Like C<function_args>, but the C<'this'> string does not apply. If 
+C<constructor_args> is
 omitted, but C<constructor> is not, the arg list will be set to
 C<[ qw( scope args ) ]>.
 
@@ -241,7 +243,7 @@ sub new { # ~~~ This sub needs some error-checking
 		;
 	}
 
-	my $self = __PACKAGE__->SUPER::new($scope);
+	my $self = $class->SUPER::new($scope);
 	my $guts = $$self;
 
 	my $global = $scope;
@@ -265,9 +267,12 @@ sub new { # ~~~ This sub needs some error-checking
 	{ no warnings 'uninitialized';
 	$$guts{function} =
 	  ref($opts{function}) =~ /^(?:JE::Code|CODE)\z/ ? $opts{function}
-	: length $opts{function} ? JE::Code::parse($scope,
-	                                           $opts{function})
-	: '';
+	: length $opts{function} ?
+		(
+		  $$guts{func_src} = $opts{function},
+		  JE::Code::parse($scope, $opts{function})
+		)
+	: ($$guts{func_src} = '');
 	} #warnings back on
 
 	$self->prop({
@@ -293,8 +298,11 @@ sub new { # ~~~ This sub needs some error-checking
 			? @{$opts{constructor_args}} : ('scope', 'args')
 		];
 	}
+	if(exists $opts{name}) {
+		$$guts{func_name} = $opts{name};
+	}
 	 	
-	bless $self, $class;
+	$self;
 }
 
 sub new_method {
@@ -309,6 +317,16 @@ sub new_method {
 Calls a function with the given arguments. The invocant (the 'this' value)
 will be the global object. This is just a wrapper around C<apply>.
 
+=cut
+
+sub call {
+	my $self = shift;
+	$self->apply($$$self{global}, @_);
+}
+
+
+
+
 =item construct
 
 Calls the constructor, if this function has one (functions written in JS
@@ -316,6 +334,45 @@ don't have this). Otherwise, an object will be created and passed to the
 function as its invocant. The return value of the function will be
 discarded, and the object (possibly modified) will be returned instead.
 
+=cut
+
+sub construct { # ~~~ we need to upgrade the args passed to construct, but 
+                #     still retain the unupgraded values to pass to the 
+                #     function *if* the function wants them downgraded
+	my $self = shift;
+	my $guts = $$self;
+	if(exists $$guts{constructor}
+	   and ref $$guts{constructor} eq 'CODE') {
+		my $code = $$guts{constructor};
+		my @args;
+		for(  @{ $$guts{constructor_args} }  ) {
+			push @args,
+			  $_ eq 'self'
+			?	$self
+			: $_ eq 'scope'
+			?	_init_scope($self, $$guts{scope},
+					[], @_)
+			: $_ eq 'args'
+			?	@_ # ~~~ downgrade if wanted
+			: $_ eq '[args]'
+			?	[@_] # ~~~ downgrade if wanted
+			: 	undef;
+		}
+		return $$guts{global}->upgrade($code->(@args));
+	}
+	else {
+		(my $obj = JE::Object->new($$guts{global}))
+		->prototype(
+			$self->prop('prototype')
+		);
+		$self->apply($obj, @_);
+		return $obj;
+	}
+}
+
+
+
+
 =item apply ( $obj, @args )
 
 Calls the function with $obj as the invocant and @args as the args.
@@ -325,10 +382,11 @@ Calls the function with $obj as the invocant and @args as the args.
 sub apply { # ~~~ we need to upgrade the args passed to apply, but still
             #     retain the unupgraded values to pass to the function *if*
             #     the function wants them downgraded
-	my ($self, $obj) = (shift, shift);
+	my ($self, $obj) = (shift, shift->to_object);
 	my $guts = $$self;
 
-	if(!UNIVERSAL::can($obj, 'primitive') or $obj->primitive) {
+	if(!blessed $obj or $obj->id eq 'null'
+	    or ref $obj eq 'JE::Object::Function::Call') {
 		$obj = $$guts{global};
 	}
 
@@ -364,7 +422,7 @@ sub apply { # ~~~ we need to upgrade the args passed to apply, but still
 sub _init_scope { # initialise the new scope for the function call
 	my($self, $scope, $argnames, @args) = @_;
 
-	bless([ @$scope, JE::Code::Call->new({
+	bless([ @$scope, JE::Object::Function::Call->new({
 		global   => $scope,
 		argnames => $argnames,
 		args     => [@args],
@@ -375,10 +433,28 @@ sub _init_scope { # initialise the new scope for the function call
 
 
 
+=item typeof
+
+This returns the string 'function'.
+
+=cut
+
+sub typeof { 'function' }
+
+
+
+
 =item class
 
 This returns the string 'Function'.
 
+=cut
+
+sub class { 'Function' }
+
+
+
+
 =item value
 
 Not yet implemented.
@@ -395,12 +471,53 @@ sub _init_proto {
 	my $scope = $$proto->{global};
 
 	# E 15.3.4
+	$proto->prop({
+		name      => 'toString',
+		value     => JE::Object::Function->new({
+			scope    => $scope,
+			name     => 'toString',
+			function_args => ['this'],
+			function => sub {
+				my $self = shift;
+				my $guts = $$self;
+				my $str = 'function ';
+				JE::String->new($scope,
+					'function ' .
+					( exists $$guts{func_name} ?
+					  $$guts{func_name} : '') .
+					'(' .
+					join(',', @{$$guts{func_argnames}})
+					. ") {" .
+					( ref $$guts{function} eq 'CODE' ?
+					  "\n    [native code]\n" :
+					  $$guts{func_src}
+					) . '}'
+				);
+			},
+		}),
+		dontenum  => 1, # ~~~ any other attrs?
+	});
+	$proto->prop({
+		name      => 'apply',
+		value     => JE::Object::Function->new({
+			scope    => $scope,
+			name     => 'apply',
+			length   => 2,
+			function_args => ['this','args'],
+			function => sub {
+				my($self,$obj,$args) = @_;
+
+				# ~~~ finish writing this with all the
+				#     'ifs' described in E 15.3.4.3
+				$self->apply($obj, eval{@{+shift}});
+			},
+		}),
+		dontenum  => 1, # ~~~ any other attrs?
+	});
 	# ~~~ $proto->prop({ ... })
 }
 
 
-1;
-
 #----------- THE REST OF THE DOCUMENTATION ---------------#
 
 =pod
@@ -422,3 +539,164 @@ sub _init_proto {
 =cut
 
 
+package JE::Object::Function::Call;
+
+our $VERSION = '0.003';
+
+sub new {
+	# See sub JE::Object::Function::_init_sub for the usage.
+
+	my($class,$opts) = @_;
+	my @args = @{$$opts{args}};
+	my %self = map {
+		my $arg_val =shift @args;
+		$_ => defined $arg_val ? $arg_val :
+			$$opts{global}->undefined;
+	} @{$$opts{argnames}};
+
+	$self{'-global'}  = $$opts{global};
+	# A call object's properties can never be accessed via bracket
+	# syntax, so '-global' cannot conflict with properties, since the
+	# latter have to be valid identifiers.
+	# ~~~ Except that I'm lax with regard to \uHHHH escapes,
+	#     so I have to make the parser more strict by disal-
+	#     lowing '-' (in escaped form) at the beginning of
+	#     an identifier.
+	
+
+	unless (exists $self{arguments}) {
+		$self{arguments} = 
+			JE::Object::Function::Arguments->new(
+				$$opts{global},
+				$$opts{function},
+				\%self,
+				$$opts{argnames},
+				@{$$opts{args}},
+			);
+	};
+
+	return bless \%self, $class;
+}
+
+sub prop {
+	my ($self, $name)  =(shift,shift);
+
+	# This very naughty line assumes it's being called by
+	# JE::Scope::new_var:
+	return if ref $name eq 'HASH';
+
+	if (@_ ) {
+		return $$self{$name} = $$self{'-global'}->upgrade(shift);
+	}
+
+	if (exists $$self{$name}) {
+		return $$self{$name};
+	}
+
+	return
+}
+
+sub delete { # ~~~ Can delete be called on a property of a call object?
+             #     If so, does the arguments object still retain that prop?
+	# 'arguments' has an attribute of 'dontdel'
+
+}
+
+
+
+
+package JE::Object::Function::Arguments;
+
+our $VERSION = '0.003';
+
+our @ISA = 'JE::Object';
+
+sub new {
+	my($class,$global,$function,$call,$argnames,@args) = @_;
+	
+	my $self = $class->SUPER::new($global);
+	my $guts = $$self;
+
+	$$guts{args_call} = $call;
+	$self->prop({
+		name => 'callee',
+		value => $function,
+		dontenum => 1,
+	});
+	$self->prop({
+		name => 'length',
+		value => scalar @args,
+		dontenum => 1,
+	});
+	$$guts{args_length} = @args; # in case the length prop
+	                              # gets changed
+
+=begin pseudocode
+
+Go through the named args one by one in reverse order.
+
+If an arg with the same name as the current one has been seen
+	Create a regular numbered property for that arg.
+Else
+	Create a magical property.
+
+=end pseudocode
+
+=cut
+
+	my (%seen,$name,$val);
+	for (reverse 0..$#$argnames) {
+		($name,$val) = ($$argnames[$_], $args[$_]);
+		if($seen{$name}++) {
+			$self->prop({
+				name => $_,
+				value => $val,
+				dontenum => 1,
+			});
+		}
+		else {
+			$$guts{args_magic}{$_} = $name;
+		}
+	}
+
+	# deal with any extra properties
+	for (@$argnames..$#args) {
+		$self->prop({
+			name => $_,
+			value => $args[$_],
+			dontenum => 1,
+		});
+	}
+
+	$self;
+}
+
+sub prop {
+	# Some properties are magically linked to properties of
+	# the call object.
+
+	my($self,$name) = @_;
+	my $guts = $$self;
+	if (exists $$guts{args_magic} and exists $$guts{args_magic}{$name})
+	{
+		return $$guts{args_call}->prop(
+			$$guts{args_magic}{$name}, @_[2..$#_]
+		);
+	}
+	SUPER::prop $self @_[1..$#_];
+}
+
+sub delete { 
+	# Magical properties are still deleteable.
+	my($self,$name) = @_;
+	my $guts = $$self;
+	if (exists $$guts{args_magic} and exists $$guts{args_magic}{$name})
+	{
+		delete $$guts{args_magic}{$name}
+	}
+	SUPER::delete $self @_[1..$#_];
+}
+
+
+
+1;
\ No newline at end of file
